# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

from Magics.macro import *

ps = output({'output_formats': ['png','ps'], 'output_name':'myview'});


coastlines = pcoast({'map_coastline_land_shade' : 'off',
	                 'map_coastline_land_shade_colour' : 'white',
					 'map_coastline_sea_shade' : 'off',
					 'map_coastline_sea_shade_colour' : 'white'})


odb = odb_geopoints({'odb_filename' : 'test.odb',
           'odb_latitude_variable' :'lat@hdr', 
           'odb_longitude_variable' : 'lon@hdr',
           'odb_value_variable': 'lat@hdr'
        })

symbol = psymb({'symbol_table_mode' : 'off', 
                'symbol_colour'     : 'blue',
                'symbol_height'     : 0.16,
                'symbol_marker'     : 15,
               'symbol_type':'marker'})
			     

legend = plegend({'legend' : 'on', 
				  'legend_text_colour':'black', 
				  'legend_display_type':'continuous'})

text = ptext({'text_justification': 'left', 
			  'text_colour':'navy'})

plot(ps, odb, symbol, coastlines,  text)



import jinja2
from xml.sax.handler import ContentHandler
from xml.sax import make_parser
from datetime import date

import sys
import json


class ObjectHandler(ContentHandler):

    generated = date.today().strftime("%Y-%m-%d")
    magics = {}
    current = ""
    basic = {
        "bool": "getBool",
        "int": "getInt",
        "float": "getDouble",
        "string": "getString",
        "stringarray": "getStringArray",
        "intarray": "getIntArray",
        "floatarray": "getDoubleArray",
        "longintarray": "getLongIntArray",
    }
    types = {
        "float": "double",
        "floatarray": "doublearray",
        "longintarray": "longintarray",
    }

    def newclass(self, attrs):
        self.current = attrs.get("action", "unknown")
        self.parameters = self.magics.get(self.current, {})

    def parameter(self, attrs):
        if attrs.get("implemented") == "no":
            return
        self.parameters.update(
            {attrs["name"]: {key: attrs[key] for key in ["from", "default"]}}
        )

    options = {
        "class": newclass,
        "parameter": parameter,
    }

    def startElement(self, name, attrs):
        if name in self.options.keys():
            self.options[name](self, attrs)

    def endElement(self, name):
        self.magics[self.current] = self.parameters


object = ObjectHandler()
saxparser = make_parser()
saxparser.setContentHandler(object)

import glob


for a in glob.glob("../src/params/*.xml"):
    datasource = open(a, "r")
    saxparser.parse(datasource)

small = {key: object.magics[key] for key in ["unknown", "pcont"]}


print json.dumps(object.magics, indent=4)



#!/usr/bin/env python
import jinja2
from xml.sax.handler import ContentHandler
from xml.sax import make_parser
from datetime import date

import sys


class ObjectHandler(ContentHandler):

    name = ""
    tag = ""
    generated = date.today().strftime("%Y-%m-%d")
    parameters = {"basic": [], "factory": []}
    basic = {
        "bool": "getBool",
        "int": "getInt",
        "float": "getDouble",
        "string": "getString",
        "stringarray": "getStringArray",
        "intarray": "getIntArray",
        "floatarray": "getDoubleArray",
        "longintarray": "getLongIntArray",
    }
    types = {
        "float": "double",
        "floatarray": "doublearray",
        "longintarray": "longintarray",
    }
    include = {}
    include_options = {}
    implements = []

    def addinterface(self, name):
        if name == "":
            return
        add = name.split("/")
        for a in add:
            self.implements.append(a)

    def addimplements(self, name):
        if name == "":
            return
        add = name.split("/")
        for a in add:
            self.include["%s.h" % a] = "%s.h" % a
        self.addinterface(name)

    def newclass(self, attrs):
        self.name = attrs.get("name")
        self.prefix = attrs.get("prefix", "").split("/")
        self.tags = attrs.get("xmltag", "").split("/")
        self.addimplements(attrs.get("implements", ""))
        self.addinterface(attrs.get("interface", ""))
        self.current = {}

    def parameter(self, attrs):
        if attrs.get("implemented") == "no":
            return
        type = attrs.get("to")
        fromt = attrs.get("from")
        to = self.types.get(type, type)

        if type in self.basic:
            self.parameters["basic"].append(
                {
                    "name": attrs.get("name"),
                    "from": self.types.get(fromt, fromt),
                    "to": to,
                    "member": attrs.get("member"),
                    "default": attrs.get("default"),
                    "method": self.basic[type],
                }
            )
            if type in ["string", "bool"]:
                self.parameters["basic"][-1]["delimiter"] = '"'
        else:

            self.parameters["factory"].append(
                {
                    "name": attrs.get("name"),
                    "from": attrs.get("from"),
                    "to": attrs.get("to"),
                    "member": attrs.get("member"),
                    "default": attrs.get("default"),
                    "options": [],
                    "method": "setMember",
                    "delimiter": '"',
                }
            )
            if type in [
                "LineStyle",
                "ArrowPosition",
                "Justification",
                "AxisAutomaticSetting",
                "ListPolicy",
                "Position",
                "Hemisphere",
                "DisplayType",
                "Matrix",
                "cairo_t*",
            ]:
                self.parameters["factory"][-1]["enum"] = True
                self.parameters["factory"][-1]["method"] = "setAttribute"
                self.parameters["factory"][-1]["niceprint"] = True

            else:
                self.include[
                    attrs.get("include", "%s.h" % attrs.get("to"))
                ] = attrs.get("include", "%s.h" % attrs.get("to"))
            if type in ["Matrix"]:
                self.parameters["factory"][-1]["delimiter"] = ""
                self.include["Matrix.h"] = "Matrix.h"
            if type in ["cairo_t*"]:
                self.parameters["factory"][-1]["delimiter"] = ""
                self.include["cairo.h"] = "cairo.h"
            if type in ["Colour"]:
                self.parameters["factory"][-1]["niceprint"] = True

    def option(self, attrs):

        param = self.parameters["factory"][-1]
        name = attrs.get("xml")
        if name != None:
            param["options"].append(
                {
                    "key": attrs.get("xml"),
                    "name": name.replace(":", "_"),
                    "parent": param["to"],
                    "object": attrs.get("name"),
                }
            )
            self.include_options[attrs["include"]] = attrs["include"]

        if attrs.get("fortran") != attrs.get("xml"):
            name = attrs.get("fortran")
            if name != None:
                param["options"].append(
                    {
                        "key": attrs.get("fortran"),
                        "name": name.replace(":", "_"),
                        "parent": param["to"],
                        "object": attrs.get("name"),
                    }
                )
                self.include_options[attrs["include"]] = attrs["include"]

    options = {"class": newclass, "parameter": parameter, "option": option}

    def startElement(self, name, attrs):
        if name in self.options.keys():
            self.options[name](self, attrs)

    def endElement(self, name):
        pass


object = ObjectHandler()
saxparser = make_parser()
saxparser.setContentHandler(object)

toolssource = sys.argv[1]

datasource = open(sys.argv[2], "r")
destination = sys.argv[3]

saxparser.parse(datasource)


with open("%s/source.template" % toolssource, "r") as source:
    template = jinja2.Template(source.read())
with open("%s/%sAttributes.cc" % (destination, object.name), "wt") as out:
    out.write(
        template.render(
            object=object.name,
            string_parameters=object.parameters["basic"],
            factory_parameters=object.parameters["factory"],
            include=object.include,
            include_options=object.include_options,
            date=object.generated,
            tags=object.tags,
            tag=object.tags[0],
            prefix=object.prefix,
        )
    )
with open("%s/header.template" % (toolssource), "r") as source:
    template = jinja2.Template(source.read())
with open("%s/%sAttributes.h" % (destination, object.name), "wt") as out:
    out.write(
        template.render(
            object=object.name,
            string_parameters=object.parameters["basic"],
            factory_parameters=object.parameters["factory"],
            include=object.include,
            include_options=object.include_options,
            implements=object.implements,
            date=object.generated,
            tags=object.tags,
            prefix=object.prefix,
        )
    )



# (C) Copyright 1996-2016 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


from xml.sax.handler import ContentHandler
from xml.sax import make_parser
import sys


class ObjectHandler(ContentHandler):
    title = ""
    name = ""
    tab1 = 0
    tab2 = 2
    open_file = 0

    def tab(self, n):
        sep = ""
        i = 0
        while i < n:
            sep = sep + "\t"
            i = i + 1
        return sep

    def nice(self, name, attrs):
        sep1 = self.tab(self.tab1)
        sep2 = self.tab(self.tab2)
        self.file.write("%s<%s " % (sep1, name))
        for key in attrs.getNames():
            self.file.write(" %s='%s'" % (key, attrs.getValue(key)))
        self.file.write(">")

    def characters(self, data):
        if self.open_file == 1:
            self.file.write(data)

    def startElement(self, name, attrs):
        print "found %s " % name
        if name == "magics":
            return
        if name == "static":
            return

        if name == "class":
            self.name = attrs.get("name")
            filename = "../params/%s.xml" % self.name
            self.file = open(filename, "w")
            self.open_file = 1
            self.file.write("<magics>\n")
            print "open %s " % filename
        self.nice(name, attrs)
        self.tab1 = self.tab1 + 1
        self.tab2 = self.tab2 + 1

    def endElement(self, name):
        print "close %s\n " % name
        if self.open_file == 0:
            return

        self.tab1 = self.tab1 - 1
        self.tab2 = self.tab2 - 1
        self.file.write("%s</%s>" % (self.tab(self.tab1), name))
        if name == "class":
            self.file.write("\n</magics>\n")
            self.file.close()
            self.open_file = 0


object = ObjectHandler()
saxparser = make_parser()
saxparser.setContentHandler(object)

datasource = open(sys.argv[1], "r")
saxparser.parse(datasource)
